@@ -1,7 +1,6 @@
 //=============================================================================
 //  MuseScore
 //  Linux Music Score Editor
-//  $Id:$
 //
 //  Copyright (C) 2009-2011 Werner Schweer and others
 //
@@ -42,11 +41,11 @@ extern bool useFactorySettings;
 ChordStyleEditor::ChordStyleEditor(QWidget* parent)
    : QDialog(parent)
       {
+      setObjectName("ChordStyleEditor");
       setupUi(this);
-      setWindowTitle(tr("MuseScore: Chord Symbols Style Editor"));
+      setWindowTitle(tr("Chord Symbols Style Editor"));
       setWindowFlags(this->windowFlags() & ~Qt::WindowContextHelpButtonHint);
 
-      fileButton->setIcon(*icons[int(Icons::fileOpen_ICON)]);
       chordList = 0;
       score = 0;
 
@@ -63,7 +62,7 @@ ChordStyleEditor::ChordStyleEditor(QWidget* parent)
 void ChordStyleEditor::setScore(Score* s)
       {
       score = s;
-      setChordList(s->style()->chordList());
+      setChordList(s->style().chordList());
       }
 
 //---------------------------------------------------------
@@ -197,11 +196,12 @@ void ChordStyleEditor::harmonyChanged(QTreeWidgetItem* current, QTreeWidgetItem*
 void ChordStyleEditor::save()
       {
       QSettings settings;
-      settings.beginGroup("ChordStyleEditor");
+      settings.beginGroup(objectName());
       settings.setValue("splitter1", splitter1->saveState());
       settings.setValue("splitter2", splitter2->saveState());
 //      settings.setValue("list", harmonyList->saveState());
       settings.setValue("col1", harmonyList->columnWidth(0));
+      MuseScore::saveGeometry(this);
       }
 
 //---------------------------------------------------------
@@ -210,9 +210,10 @@ void ChordStyleEditor::save()
 
 void ChordStyleEditor::restore()
       {
+      MuseScore::restoreGeometry(this);
       if (!useFactorySettings) {
             QSettings settings;
-            settings.beginGroup("ChordStyleEditor");
+            settings.beginGroup(objectName());
             splitter1->restoreState(settings.value("splitter1").toByteArray());
             splitter2->restoreState(settings.value("splitter2").toByteArray());
             harmonyList->setColumnWidth(0, settings.value("col1", 30).toInt());
@@ -275,13 +276,13 @@ void HarmonyCanvas::paintEvent(QPaintEvent* event)
             p.drawText(ts->x, ts->y, ts->text);
             }
 
-      if (dragElement && dragElement->type() == Element::Type::FSYMBOL) {
+      if (dragElement && dragElement->type() == ElementType::FSYMBOL) {
             FSymbol* sb = static_cast<FSymbol*>(dragElement);
 
-            double _spatium = 2.0 * PALETTE_SPATIUM / extraMag;
-            const TextStyle* st = &gscore->textStyle(TextStyleType::HARMONY);
-            QFont ff(st->fontPx(_spatium));
-            ff.setFamily(sb->font().family());
+//TODO:ws             double _spatium = 2.0 * PALETTE_SPATIUM / extraMag;
+//            const TextStyle* st = &gscore->textStyle(TextStyleType::HARMONY);
+//            QFont ff(st->font(_spatium * MScore::pixelRatio));
+//            ff.setFamily(sb->font().family());
 
             QString s;
             int code = sb->code();
@@ -291,7 +292,7 @@ void HarmonyCanvas::paintEvent(QPaintEvent* event)
                   }
             else
                   s = QChar(code);
-            p.setFont(ff);
+//TODO:ws            p.setFont(ff);
             QPen pen(Qt::yellow);
             p.setPen(pen);
             p.drawText(dragElement->pos(), s);
@@ -302,8 +303,9 @@ void HarmonyCanvas::paintEvent(QPaintEvent* event)
 //   render
 //---------------------------------------------------------
 
-void HarmonyCanvas::render(const QList<RenderAction>& renderList, double& x, double& y, int tpc, NoteSpellingType noteSpelling, NoteCaseType noteCase)
+void HarmonyCanvas::render(const QList<RenderAction>& /*renderList*/, double& /*x*/, double& /*y*/, int /*tpc*/, NoteSpellingType /*noteSpelling*/, NoteCaseType /*noteCase*/)
       {
+#if 0 // TODO:ws
       QStack<QPointF> stack;
       int fontIdx = 0;
       double _spatium = 2.0 * PALETTE_SPATIUM / extraMag;
@@ -314,15 +316,15 @@ void HarmonyCanvas::render(const QList<RenderAction>& renderList, double& x, dou
 
       foreach(ChordFont cf, chordList->fonts) {
             if (cf.family.isEmpty() || cf.family == "default")
-                  fontList.append(st->fontPx(_spatium * cf.mag));
+                  fontList.append(st->font(_spatium * cf.mag * MScore::pixelRatio));
             else {
-                  QFont ff(st->fontPx(_spatium * cf.mag));
+                  QFont ff(st->font(_spatium * cf.mag * MScore::pixelRatio));
                   ff.setFamily(cf.family);
                   fontList.append(ff);
                   }
             }
       if (fontList.isEmpty())
-            fontList.append(st->fontPx(_spatium));
+            fontList.append(st->font(_spatium * MScore::pixelRatio));
 
       foreach(const RenderAction& a, renderList) {
             if (a.type == RenderAction::RenderActionType::SET) {
@@ -388,6 +390,7 @@ void HarmonyCanvas::render(const QList<RenderAction>& renderList, double& x, dou
                         }
                   }
             }
+#endif
       }
 
 //---------------------------------------------------------
@@ -464,14 +467,15 @@ void HarmonyCanvas::setChordDescription(ChordDescription* sd, ChordList* sl)
 //   dropEvent
 //---------------------------------------------------------
 
-void HarmonyCanvas::dropEvent(QDropEvent* event)
+void HarmonyCanvas::dropEvent(QDropEvent* /*event*/)
       {
-      if (dragElement && dragElement->type() == Element::Type::FSYMBOL) {
+#if 0       // TODO:ws
+      if (dragElement && dragElement->type() == ElementType::FSYMBOL) {
             FSymbol* sb = static_cast<FSymbol*>(dragElement);
 
             double _spatium = 2.0 * PALETTE_SPATIUM / extraMag;
             const TextStyle* st = &gscore->textStyle(TextStyleType::HARMONY);
-            QFont ff(st->fontPx(_spatium));
+            QFont ff(st->font(_spatium * MScore::pixelRatio));
             ff.setFamily(sb->font().family());
 
 //            qDebug("drop %s", dragElement->name());
@@ -493,6 +497,7 @@ void HarmonyCanvas::dropEvent(QDropEvent* event)
             dragElement = 0;
             update();
             }
+#endif
       }
 
 //---------------------------------------------------------
@@ -501,16 +506,16 @@ void HarmonyCanvas::dropEvent(QDropEvent* event)
 
 void HarmonyCanvas::dragEnterEvent(QDragEnterEvent* event)
       {
-      const QMimeData* data = event->mimeData();
-      if (data->hasFormat(mimeSymbolFormat)) {
-            QByteArray a = data->data(mimeSymbolFormat);
+      const QMimeData* dta = event->mimeData();
+      if (dta->hasFormat(mimeSymbolFormat)) {
+            QByteArray a = dta->data(mimeSymbolFormat);
 
             XmlReader e(a);
 
             QPointF dragOffset;
             Fraction duration;
-            Element::Type type = Element::readType(e, &dragOffset, &duration);
-            if (type == Element::Type::FSYMBOL) {
+            ElementType type = Element::readType(e, &dragOffset, &duration);
+            if (type == ElementType::FSYMBOL) {
                   event->acceptProposedAction();
                   dragElement = Element::create(type, gscore);
                   dragElement->read(e);
@@ -537,7 +542,7 @@ void HarmonyCanvas::dragLeaveEvent(QDragLeaveEvent*)
 void HarmonyCanvas::dragMoveEvent(QDragMoveEvent* event)
       {
       event->acceptProposedAction();
-      if (dragElement && dragElement->type() == Element::Type::FSYMBOL) {
+      if (dragElement && dragElement->type() == ElementType::FSYMBOL) {
             dragElement->setPos(imatrix.map(event->pos()));
             update();
             }
@@ -561,7 +566,7 @@ void HarmonyCanvas::deleteAction()
 
 static void updateHarmony(void*, Element* e)
       {
-      if (e->type() == Element::Type::HARMONY)
+      if (e->type() == ElementType::HARMONY)
             static_cast<Harmony*>(e)->render();
       }
 
@@ -572,7 +577,7 @@ static void updateHarmony(void*, Element* e)
 void ChordStyleEditor::accept()
       {
       canvas->updateChordDescription();
-      score->style()->setChordList(chordList);
+      score->style().setChordList(chordList);
       chordList = 0;
 
       score->scanElements(0, updateHarmony);
@@ -594,7 +599,7 @@ void HarmonyCanvas::updateChordDescription()
       double x  = 0, y = 0;
       foreach(const TextSegment* ts, textList) {
             ++idx;
-            if (idx == 1) {     // dont save base
+            if (idx == 1) {     // don’t save base
                   x = ts->x + ts->width();
                   y = ts->y;
                   continue;
