@@ -14,12 +14,15 @@
 #define __LOGINDIALOG_H__
 
 #include "ui_logindialog.h"
-#include "loginmanager.h"
 
 namespace Ms {
 
+class LoginManager;
+
 //---------------------------------------------------------
 //   LoginDialog
+//    Old-style login dialog in case QtWebEngine is
+//    unavailable.
 //---------------------------------------------------------
 
 class LoginDialog : public QDialog, public Ui::LoginDialog
@@ -28,6 +31,8 @@ class LoginDialog : public QDialog, public Ui::LoginDialog
       
       LoginManager* _loginManager;
 
+      virtual void hideEvent(QHideEvent*);
+
    signals:
       void loginSuccessful();
       
