@@ -1,7 +1,6 @@
 //=============================================================================
 //  MuseScore
 //  Linux Music Score Editor
-//  $Id: magbox.cpp 5568 2012-04-22 10:08:43Z wschweer $
 //
 //  Copyright (C) 2002-2008 Werner Schweer and others
 //
@@ -68,6 +67,7 @@ MagBox::MagBox(QWidget* parent)
       setToolTip(tr("Zoom"));
       setWhatsThis(tr("Zoom"));
       setValidator(new MagValidator(this));
+      setAutoCompletion(false);
 
       int i = 0;
       for (const MagEntry& e : magTable) {
@@ -81,7 +81,7 @@ MagBox::MagBox(QWidget* parent)
       addItem(QString("%1%").arg(freeMag * 100), int(MagIdx::MAG_FREE));
       setFocusPolicy(Qt::StrongFocus);
       setAccessibleName(tr("Zoom"));
-      setFixedHeight(preferences.iconHeight + 10);  // hack
+      setFixedHeight(preferences.getInt(PREF_UI_THEME_ICONHEIGHT) + 8);  // hack
       connect(this, SIGNAL(currentIndexChanged(int)), SLOT(indexChanged(int)));
       connect(lineEdit(), SIGNAL(returnPressed()), SLOT(textChanged()));
       }
@@ -140,7 +140,8 @@ double MagBox::getMag(ScoreView* canvas) const
       qreal pmag           = mscore->physicalDotsPerInch() / DPI;
       double cw            = canvas->width();
       double ch            = canvas->height();
-      const PageFormat* pf = score->pageFormat();
+      qreal pw             = score->styleD(Sid::pageWidth);
+      qreal ph             = score->styleD(Sid::pageHeight);
       double nmag;
 
       switch (idx) {
@@ -155,13 +156,13 @@ double MagBox::getMag(ScoreView* canvas) const
             case MagIdx::MAG_1600:    nmag = 16.0 * pmag; break;
 
             case MagIdx::MAG_PAGE_WIDTH:      // page width
-                  nmag = cw / (pf->width() * DPI);
+                  nmag = cw / (pw * DPI);
                   break;
 
             case MagIdx::MAG_PAGE:     // page
                   {
-                  double mag1 = cw / (pf->width() *  DPI);
-                  double mag2 = ch / (pf->height() * DPI);
+                  double mag1 = cw / (pw *  DPI);
+                  double mag2 = ch / (ph * DPI);
                   nmag  = (mag1 > mag2) ? mag2 : mag1;
                   }
                   break;
@@ -171,12 +172,12 @@ double MagBox::getMag(ScoreView* canvas) const
                   double mag1 = 0;
                   double mag2 = 0;
                   if (MScore::verticalOrientation()) {
-                        mag1 = ch / (pf->height() * 2 * DPI +  MScore::verticalPageGap);
-                        mag2 = cw / (pf->width() * DPI);
+                        mag1 = ch / (ph * 2 * DPI +  MScore::verticalPageGap);
+                        mag2 = cw / (pw * DPI);
                         }
                   else {
-                        mag1 = cw / (pf->width() * 2 * DPI + 50);
-                        mag2 = ch / (pf->height() * DPI);
+                        mag1 = cw / (pw * 2 * DPI + 50);
+                        mag2 = ch / (ph * DPI);
                         }
                   nmag  = (mag1 > mag2) ? mag2 : mag1;
                   }
