@@ -1,9 +1,8 @@
 //=============================================================================
 //  MuseScore
 //  Music Composition & Notation
-//  $Id: musescore.h 5657 2012-05-21 15:46:06Z lasconic $
 //
-//  Copyright (C) 2002-2011 Werner Schweer and others
+//  Copyright (C) 2002-2016 Werner Schweer and others
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License version 2.
@@ -27,15 +26,16 @@
 #include "ui_insertmeasuresdialog.h"
 #include "ui_aboutbox.h"
 #include "ui_aboutmusicxmlbox.h"
-#include "ui_startdialog.h"
 #include "singleapp/src/QtSingleApplication"
 #include "updatechecker.h"
-#include "loginmanager.h"
-#include "uploadscoredialog.h"
 #include "libmscore/musescoreCore.h"
+#include "libmscore/score.h"
+#include "newwizard.h"
 
 namespace Ms {
 
+class UploadScoreDialog;
+class LoginManager;
 class Shortcut;
 class ScoreView;
 class Element;
@@ -48,6 +48,7 @@ class PlayPanel;
 class Mixer;
 class Debugger;
 class MeasureListEditor;
+class MasterScore;
 class Score;
 class Tuplet;
 class PageSettings;
@@ -55,7 +56,7 @@ class PaletteBox;
 class Palette;
 class PaletteScrollArea;
 class SelectionWindow;
-class Xml;
+class XmlWriter;
 class MagBox;
 class NewWizard;
 class ExcerptsDialog;
@@ -71,9 +72,11 @@ class ScriptEngine;
 class KeyEditor;
 class ChordStyleEditor;
 class Navigator;
+class Timeline;
 class PianoTools;
 class MediaDialog;
 class Workspace;
+class WorkspaceDialog;
 class AlbumManager;
 class WebPageDockWidget;
 class ChordList;
@@ -81,16 +84,27 @@ class Capella;
 class Inspector;
 class OmrPanel;
 class NScrollArea;
+class TDockWidget;
 class Sym;
 class MasterPalette;
 class PluginCreator;
+#ifdef SCRIPT_INTERFACE
 class PluginManager;
+class QmlPluginEngine;
+#endif
 class MasterSynthesizer;
+class SynthesizerState;
 class Driver;
 class Seq;
 class ImportMidiPanel;
+class ScoreComparisonTool;
+class ScriptRecorder;
+class ScriptRecorderWidget;
 class Startcenter;
 class HelpBrowser;
+class ToolbarEditor;
+class TourHandler;
+class GeneralAutoUpdater;
 
 struct PluginDescription;
 enum class SelState : char;
@@ -163,6 +177,8 @@ class AboutMusicXMLBoxDialog : public QDialog, Ui::AboutMusicXMLBox {
 class InsertMeasuresDialog : public QDialog, public Ui::InsertMeasuresDialogBase {
       Q_OBJECT
 
+      virtual void hideEvent(QHideEvent*);
+
    private slots:
       virtual void accept();
 
@@ -185,21 +201,6 @@ class MeasuresDialog : public QDialog, public Ui::MeasuresDialogBase {
       };
 
 
-//---------------------------------------------------------
-//   StartDialog
-//---------------------------------------------------------
-
-class StartDialog : public QDialog, public Ui::StartDialog {
-      Q_OBJECT
-
-   private slots:
-      void createScoreClicked();
-      void loadScoreClicked();
-
-   public:
-      StartDialog(QWidget* parent = 0);
-      };
-
 //---------------------------------------------------------
 //   MuseScoreApplication (mac only)
 //---------------------------------------------------------
@@ -207,12 +208,13 @@ class StartDialog : public QDialog, public Ui::StartDialog {
 class MuseScoreApplication : public QtSingleApplication {
    public:
       QStringList paths;
-      MuseScoreApplication(const QString &id, int &argc, char **argv)
+      MuseScoreApplication(const QString& id, int &argc, char **argv)
          : QtSingleApplication(id, argc, argv) {
             };
       virtual bool event(QEvent *ev) override;
       };
 
+
 //---------------------------------------------------------
 //   MuseScore
 //---------------------------------------------------------
@@ -222,31 +224,59 @@ class MuseScore : public QMainWindow, public MuseScoreCore {
 
       QSettings settings;
       ScoreView* cv                        { 0 };
+      ScoreTab* ctab                       { 0 };
+      QMap<MasterScore*, bool> scoreWasShown; // whether each score in scoreList has ever been shown
       ScoreState _sstate;
       UpdateChecker* ucheck;
+      ExtensionsUpdateChecker* packUChecker = nullptr;
+
+      static const std::list<const char*> _allNoteInputMenuEntries;
+      std::list<const char*> _noteInputMenuEntries { _allNoteInputMenuEntries };
+
+      static const std::list<const char*> _allFileOperationEntries;
+      std::list<const char*> _fileOperationEntries { _allFileOperationEntries };
+
+      static const std::list<const char*> _allPlaybackControlEntries;
+      std::list<const char*> _playbackControlEntries { _allPlaybackControlEntries };
+
+      bool _playPartOnly = true; // play part only vs. full score
 
       QVBoxLayout* layout;    // main window layout
       QSplitter* splitter;
       ScoreTab* tab1;
       ScoreTab* tab2;
       NScrollArea* _navigator;
+      TDockWidget* _timeline;
       ImportMidiPanel* importmidiPanel     { 0 };
       QFrame* importmidiShowPanel;
       QSplitter* mainWindow;
 
-      QMenu* menuView;
-      QMenu* openRecent;
+      ScoreComparisonTool* scoreCmpTool    { 0 };
+      ScriptRecorderWidget* scriptRecorder { nullptr };
 
       MagBox* mag;
       QComboBox* viewModeCombo;
       QAction* playId;
 
+      QAction* pref;
+      QAction* onlineHandbookAction;
+      QAction* aboutAction;
+      QAction* aboutQtAction;
+      QAction* aboutMusicXMLAction;
+      QAction* checkForUpdateAction        { 0 };
+      QAction* askForHelpAction;
+      QAction* reportBugAction;
+      QAction* leaveFeedbackAction;
+      QAction* revertToFactoryAction;
+
       QProgressBar* _progressBar           { 0 };
       PreferenceDialog* preferenceDialog   { 0 };
       QToolBar* cpitchTools;
+      QToolBar* fotoTools;
       QToolBar* fileTools;
       QToolBar* transportTools;
       QToolBar* entryTools;
+      QToolBar* feedbackTools;
       TextTools* _textTools                { 0 };
       PianoTools* _pianoTools              { 0 };
       MediaDialog* _mediaDialog            { 0 };
@@ -257,14 +287,40 @@ class MuseScore : public QMainWindow, public MuseScoreCore {
       InsertMeasuresDialog* insertMeasuresDialog { 0 };
       MasterPalette* masterPalette         { 0 };
       PluginCreator* _pluginCreator        { 0 };
+#ifdef SCRIPT_INTERFACE
       PluginManager* pluginManager         { 0 };
+      QmlPluginEngine* _qmlEngine          { 0 };
+#endif
       SelectionWindow* selectionWindow     { 0 };
 
-      QMenu* _fileMenu;
+      QMenu* menuFile;
+      QMenu* openRecent;
       QMenu* menuEdit;
-      QMenu* menuNotes;
-      QMenu* menuLayout;
-      QMenu* menuStyle;
+      QMenu* menuView;
+      QMenu* menuToolbars;
+      QMenu* menuWorkspaces;
+
+      QMenu* menuAdd;
+      QMenu* menuAddMeasures;
+      QMenu* menuAddFrames;
+      QMenu* menuAddText;
+      QMenu* menuAddLines;
+      QMenu* menuAddPitch;
+      QMenu* menuAddInterval;
+      QMenu* menuTuplet;
+
+      QMenu* menuFormat;
+      QMenu* menuStretch;
+      QMenu* menuTools;
+      QMenu* menuVoices;
+      QMenu* menuMeasure;
+
+      QMenu* menuPlugins;
+      QMenu* menuHelp;
+      QMenu* menuTours;
+#ifndef NDEBUG
+      QMenu* menuDebug;
+#endif
       AlbumManager* albumManager           { 0 };
 
       QWidget* _searchDialog               { 0 };
@@ -293,6 +349,9 @@ class MuseScore : public QMainWindow, public MuseScoreCore {
       PaletteBox* paletteBox         { 0 };
       Inspector* _inspector          { 0 };
       OmrPanel* omrPanel             { 0 };
+      QWidget* lastFocusWidget       { 0 };
+
+      QPushButton* showMidiImportButton {0};
 
       bool _midiinEnabled            { true };
       QList<QString> plugins;
@@ -303,9 +362,7 @@ class MuseScore : public QMainWindow, public MuseScoreCore {
       void removeMenuEntry(PluginDescription*);
 
       QTimer* autoSaveTimer;
-      QList<QAction*> qmlPluginActions;
       QList<QAction*> pluginActions;
-      QSignalMapper* pluginMapper        { 0 };
 
       PianorollEditor* pianorollEditor   { 0 };
       DrumrollEditor* drumrollEditor     { 0 };
@@ -330,7 +387,6 @@ class MuseScore : public QMainWindow, public MuseScoreCore {
       QFileDialog* saveImageDialog       { 0 };
       QFileDialog* loadChordStyleDialog  { 0 };
       QFileDialog* saveChordStyleDialog  { 0 };
-//      QFileDialog* loadSoundFontDialog   { 0 };
       QFileDialog* loadSfzFileDialog     { 0 };
       QFileDialog* loadBackgroundDialog  { 0 };
       QFileDialog* loadScanDialog        { 0 };
@@ -342,22 +398,24 @@ class MuseScore : public QMainWindow, public MuseScoreCore {
       QFileDialog* saveDrumsetDialog     { 0 };
       QFileDialog* savePluginDialog      { 0 };
 
+      WorkspaceDialog* _workspaceDialog   { 0 };
+
       QDialog* editRasterDialog          { 0 };
 
       QAction* hRasterAction;
       QAction* vRasterAction;
 
-      QMenu* menuWorkspaces;
+      ToolbarEditor* editToolbars        { 0 };
       QActionGroup* workspaces           { 0 };
 
       bool inChordEditor                 { false };
 
       QComboBox* layerSwitch;
       QComboBox* playMode;
-      QNetworkAccessManager* networkManager { 0 };
-      QAction* lastCmd                      { 0 };
-      const Shortcut* lastShortcut          { 0 };
-      QHelpEngine* _helpEngine              { 0 };
+      QNetworkAccessManager* _networkManager { 0 };
+      QAction* lastCmd                       { 0 };
+      const Shortcut* lastShortcut           { 0 };
+      QHelpEngine* _helpEngine               { 0 };
       int globalX, globalY;       // current mouse position
 
       QAction* countInAction;
@@ -373,11 +431,22 @@ class MuseScore : public QMainWindow, public MuseScoreCore {
 
       qreal _physicalDotsPerInch;
 
+      QMessageBox* infoMsgBox;
+      TourHandler* _tourHandler { 0 };
+
+      std::unique_ptr<GeneralAutoUpdater> autoUpdater;
+
       //---------------------
 
       virtual void closeEvent(QCloseEvent*);
       virtual void dragEnterEvent(QDragEnterEvent*);
       virtual void dropEvent(QDropEvent*);
+      virtual void changeEvent(QEvent *e);
+      virtual void showEvent(QShowEvent *event);
+
+      void retranslate();
+      void setMenuTitles();
+      void updateMenu(QMenu*& menu, QString menu_id, QString objectName);
 
       void playVisible(bool flag);
       void launchBrowser(const QString whereTo);
@@ -387,7 +456,7 @@ class MuseScore : public QMainWindow, public MuseScoreCore {
       void symbolMenu();
       void showKeyEditor();
       bool saveFile();
-      bool saveFile(Score* score);
+      bool saveFile(MasterScore* score);
       void fingeringMenu();
 
       int  pluginIdxFromPath(QString pluginPath);
@@ -398,8 +467,10 @@ class MuseScore : public QMainWindow, public MuseScoreCore {
       void showInspector(bool);
       void showOmrPanel(bool);
       void showNavigator(bool);
+      void showTimeline(bool);
       void showSelectionWindow(bool);
       void showSearchDialog();
+      void showToolbarEditor();
       void splitWindow(bool horizontal);
       void removeSessionFile();
       void editChordStyle();
@@ -416,7 +487,20 @@ class MuseScore : public QMainWindow, public MuseScoreCore {
       void showModeText(const QString&);
       void addRecentScore(const QString& scorePath);
 
-      void updateNewWizard();
+      void updateViewModeCombo();
+      void switchLayoutMode(LayoutMode);
+      void setPlayRepeats(bool repeat);
+
+      ScoreTab* createScoreTab();
+      void askResetOldScorePositions(Score* score);
+
+      QString getUtmParameters(QString medium) const;
+
+      void checkForUpdatesNoUI();
+
+   signals:
+      void windowSplit(bool);
+      void musescoreWindowWasShown();
 
    private slots:
       void cmd(QAction* a, const QString& cmd);
@@ -426,12 +510,13 @@ class MuseScore : public QMainWindow, public MuseScoreCore {
       void about();
       void aboutQt();
       void aboutMusicXML();
-      void reportBug();
+      void reportBug(QString medium);
       void askForHelp();
+      void leaveFeedback(QString medium);
       void openRecentMenu();
       void selectScore(QAction*);
       void startPreferenceDialog();
-      void preferencesChanged();
+      void preferencesChanged(bool fromWorkspace = false);
       void seqStarted();
       void seqStopped();
       void cmdAppendMeasures();
@@ -441,6 +526,14 @@ class MuseScore : public QMainWindow, public MuseScoreCore {
       void removeTab(int);
       void removeTab();
       void clipboardChanged();
+      void inputMethodAnchorRectangleChanged();
+      void inputMethodAnimatingChanged();
+      void inputMethodCursorRectangleChanged();
+      void inputMethodInputDirectionChanged(Qt::LayoutDirection newDirection);
+      void inputMethodInputItemClipRectangleChanged();
+      void inputMethodKeyboardRectangleChanged();
+      void inputMethodLocaleChanged();
+      void inputMethodVisibleChanged();
       void endSearch();
       void saveScoreDialogFilterSelected(const QString&);
 #ifdef OSC
@@ -463,39 +556,51 @@ class MuseScore : public QMainWindow, public MuseScoreCore {
       void showWorkspaceMenu();
       void switchLayer(const QString&);
       void switchPlayMode(int);
-      void networkFinished(QNetworkReply*);
+      void networkFinished();
       void switchLayoutMode(int);
       void showMidiImportPanel();
       void changeWorkspace(QAction*);
+      void onLongOperationFinished();
+
+      virtual QMenu* createPopupMenu() override;
+
+      QByteArray exportPdfAsJSON(Score*);
 
    public slots:
       virtual void cmd(QAction* a);
       void dirtyChanged(Score*);
-      void setPos(int tick);
+      void setPos(const Fraction& tick);
       void pluginTriggered(int);
+      void pluginTriggered(QString path);
       void handleMessage(const QString& message);
       void setCurrentScoreView(ScoreView*);
       void setCurrentScoreView(int);
+      void setCurrentScores(Score* s1, Score* s2 = nullptr);
       void setNormalState()    { changeState(STATE_NORMAL); }
       void setPlayState()      { changeState(STATE_PLAY); }
-      void checkForUpdate();
-      QMenu* fileMenu() const  { return _fileMenu; }
+      void setNoteEntryState() { changeState(STATE_NOTE_ENTRY); }
+      void checkForUpdatesUI();
+      void checkForExtensionsUpdate();
       void midiNoteReceived(int channel, int pitch, int velo);
       void midiNoteReceived(int pitch, bool ctrl, int velo);
       void instrumentChanged();
       void showMasterPalette(const QString& = 0);
       void selectionChanged(SelState);
       void createNewWorkspace();
-      void changeWorkspace(Workspace* p);
+      void editWorkspace();
+      void changeWorkspace(Workspace* p, bool first=false);
+      void mixerPreferencesChanged(bool showMidiControls);
+      void checkForUpdates();
+      void restartAudioEngine();
 
    public:
       MuseScore();
       ~MuseScore();
-      bool checkDirty(Score*);
+      bool checkDirty(MasterScore*);
       PlayPanel* getPlayPanel() const { return playPanel; }
       Mixer* getMixer() const { return mixer; }
       QMenu* genCreateMenu(QWidget* parent = 0);
-      virtual int appendScore(Score*);
+      virtual int appendScore(MasterScore*);
       void midiCtrlReceived(int controller, int value);
       void showElementContext(Element* el);
       void cmdAppendMeasures(int);
@@ -520,7 +625,8 @@ class MuseScore : public QMainWindow, public MuseScoreCore {
       void showPluginCreator(QAction*);
       void showPluginManager();
 
-      void updateTabNames();
+//      void updateTabNames();
+      void updatePaletteBeamMode(bool unselect = false);
       QProgressBar* showProgressBar();
       void hideProgressBar();
       void addRecentScore(Score*);
@@ -530,46 +636,56 @@ class MuseScore : public QMainWindow, public MuseScoreCore {
       QString lastSaveCopyDirectory;
       QString lastSaveCopyFormat;
       QString lastSaveDirectory;
+      QString lastSaveCaptureName;
       SynthControl* getSynthControl() const       { return synthControl; }
-      void editInPianoroll(Staff* staff);
+      void editInPianoroll(Staff* staff, Position* p = 0);
       void editInDrumroll(Staff* staff);
       PianorollEditor* getPianorollEditor() const { return pianorollEditor; }
       DrumrollEditor* getDrumrollEditor() const   { return drumrollEditor; }
       PianoTools* pianoTools() const              { return _pianoTools; }
+#ifdef SCRIPT_INTERFACE
+      PluginManager* getPluginManager() const     { return pluginManager; }
+      QmlPluginEngine* getPluginEngine();
+#endif
       void writeSessionFile(bool);
       bool restoreSession(bool);
       bool splitScreen() const { return _splitScreen; }
+      void setSplitScreen(bool val);
       virtual void setCurrentView(int tabIdx, int idx);
       void loadPlugins();
       void unloadPlugins();
+      void addPluginMenuEntries();
 
       ScoreState state() const { return _sstate; }
       void changeState(ScoreState);
       void updateInputState(Score*);
+      void updateShadowNote();
 
       bool readLanguages(const QString& path);
       void setRevision(QString& r)  {rev = r;}
       Q_INVOKABLE QString revision()            {return rev;}
       Q_INVOKABLE QString version()            {return VERSION;}
       Q_INVOKABLE void newFile();
+      MasterScore* getNewFile();
       Q_INVOKABLE void loadFile(const QString& url);
       void loadFile(const QUrl&);
-      virtual Score* openScore(const QString& fn);
+      QTemporaryFile* getTemporaryScoreFileCopy(const QFileInfo& info, const QString& baseNameTemplate);
+      QNetworkAccessManager* networkManager();
+      virtual Score* openScore(const QString& fn, bool switchTab = true);
       bool hasToCheckForUpdate();
+      bool hasToCheckForExtensionsUpdate();
       static bool unstable();
       bool eventFilter(QObject *, QEvent *);
       void setMidiRecordId(int id) { _midiRecordId = id; }
       int midiRecordId() const { return _midiRecordId; }
-      void setAdvancedPalette();
-      void setBasicPalette();
-      void excerptsChanged(Score*);
+      void setDefaultPalette();
       void scorePageLayoutChanged();
-      bool processMidiRemote(MidiRemoteType type, int data);
+      bool processMidiRemote(MidiRemoteType type, int data, int value);
       ScoreTab* getTab1() const { return tab1; }
       ScoreTab* getTab2() const { return tab2; }
       QList<LanguageItem>& languages() { return _languages; }
 
-      QStringList getOpenScoreNames(const QString& filter, const QString& title);
+      QStringList getOpenScoreNames(const QString& filter, const QString& title, bool singleFile = false);
       QString getSaveScoreName(const QString& title, QString& name, const QString& filter, bool folder = false);
       QString getStyleFilename(bool open, const QString& title = QString());
       QString getFotoFilename(QString& filter, QString *selectedFilter);
@@ -600,33 +716,56 @@ class MuseScore : public QMainWindow, public MuseScoreCore {
       void exportFile();
       bool exportParts();
       virtual bool saveAs(Score*, bool saveCopy, const QString& path, const QString& ext);
+      QString saveFilename(QString fn);
       bool savePdf(const QString& saveName);
       bool savePdf(Score* cs, const QString& saveName);
       bool savePdf(QList<Score*> cs, const QString& saveName);
+      bool savePdf(Score* cs, QPrinter& printer);
 
 
-      Score* readScore(const QString& name);
+      MasterScore* readScore(const QString& name);
 
       bool saveAs(Score*, bool saveCopy = false);
       bool saveSelection(Score*);
       void addImage(Score*, Element*);
 
-      bool savePng(Score*, const QString& name, bool screenshot, bool transparent, double convDpi, int trimMargin, QImage::Format format);
+      bool saveAudio(Score*, QIODevice*, std::function<bool(float)> updateProgress = nullptr);
       bool saveAudio(Score*, const QString& name);
+      bool canSaveMp3();
       bool saveMp3(Score*, const QString& name);
+      bool saveMp3(Score*, QIODevice*, bool& wasCanceled);
       bool saveSvg(Score*, const QString& name);
+      bool saveSvg(Score*, QIODevice*, int pageNum = 0);
+      bool savePng(Score*, QIODevice*, int pageNum = 0);
       bool savePng(Score*, const QString& name);
-//      bool saveLilypond(Score*, const QString& name);
-      bool saveMidi(Score* score, const QString& name);
+      bool saveMidi(Score*, const QString& name);
+      bool saveMidi(Score*, QIODevice*);
+      bool savePositions(Score*, const QString& name, bool segments);
+      bool savePositions(Score*, QIODevice*, bool segments);
+      bool saveMetadataJSON(Score*, const QString& name);
+      QJsonObject saveMetadataJSON(Score*);
 
+      /////The methods are used in the musescore.com backend
+      bool exportAllMediaFiles(const QString& inFilePath, const QString& outFilePath = "/dev/stdout");
+      bool exportMp3AsJSON(const QString& inFilePath, const QString& outFilePath = "/dev/stdout");
+      bool exportPartsPdfsToJSON(const QString& inFilePath, const QString& outFilePath = "/dev/stdout");
+      /////////////////////////////////////////////////
+
+      void scoreUnrolled(MasterScore* original);
+      
       virtual void closeScore(Score* score);
 
       void addTempo();
       void addMetronome();
 
+      SynthesizerState synthesizerState() const;
+      static Synthesizer* synthesizer(const QString& name);
+
       Q_INVOKABLE QString getLocaleISOCode() const;
       Navigator* navigator() const;
       NScrollArea* navigatorScrollArea() const { return _navigator; }
+      Timeline* timeline() const;
+      TDockWidget* timelineScrollArea() const { return _timeline; }
       QWidget*   searchDialog() const;
       SelectionWindow* getSelectionWindow() const { return selectionWindow; }
       void updateLayer();
@@ -636,49 +775,60 @@ class MuseScore : public QMainWindow, public MuseScoreCore {
       bool countIn() const           { return countInAction->isChecked(); }
       bool panDuringPlayback() const { return panAction->isChecked(); }
       void noteTooShortForTupletDialog();
-      void loadFiles();
+      void loadFiles(bool switchTab = true, bool singleFile = false);
                   // midi panel functions
       void midiPanelOnSwitchToFile(const QString &file);
       void midiPanelOnCloseFile(const QString &file);
       void allowShowMidiPanel(const QString &file);
       void setMidiReopenInProgress(const QString &file);
 
-      static Palette* newTempoPalette();
-      static Palette* newTextPalette();
+      static Palette* newTempoPalette(bool defaultPalette = false);
+      static Palette* newTextPalette(bool defaultPalette = false);
       static Palette* newTimePalette();
       static Palette* newRepeatsPalette();
       static Palette* newBreaksPalette();
-      static Palette* newBeamPalette(bool basic);
-      static Palette* newDynamicsPalette(bool basic, bool master = false);
+      static Palette* newBeamPalette();
+      static Palette* newDynamicsPalette(bool defaultPalette = false);
       static Palette* newFramePalette();
       static Palette* newFingeringPalette();
       static Palette* newTremoloPalette();
       static Palette* newNoteHeadsPalette();
-      static Palette* newArticulationsPalette(bool basic);
+      static Palette* newArticulationsPalette();
+      static Palette* newOrnamentsPalette();
+      static Palette* newAccordionPalette();
       static Palette* newBracketsPalette();
       static Palette* newBreathPalette();
       static Palette* newArpeggioPalette();
-      static Palette* newClefsPalette(bool basic);
-      static Palette* newGraceNotePalette(bool basic);
+      static Palette* newClefsPalette(bool defaultPalette = false);
+      static Palette* newGraceNotePalette();
       static Palette* newBagpipeEmbellishmentPalette();
-      static Palette* newKeySigPalette(bool basic = false);
-      static Palette* newAccidentalsPalette(bool basic = false);
-      static Palette* newBarLinePalette(bool basic);
-      static Palette* newLinesPalette(bool basic);
+      static Palette* newKeySigPalette();
+      static Palette* newAccidentalsPalette(bool defaultPalette = false);
+      static Palette* newBarLinePalette();
+      static Palette* newLinesPalette();
+      static Palette* newFretboardDiagramPalette();
+
+      WorkspaceDialog* workspaceDialog() { return _workspaceDialog; }
+      void updateIcons();
+      void updateMenus();
 
       Inspector* inspector()           { return _inspector; }
       PluginCreator* pluginCreator()   { return _pluginCreator; }
       ScoreView* currentScoreView() const { return cv; }
+      ScoreTab* currentScoreTab() const { return ctab; }
       QToolButton* playButton()        { return _playButton;    }
       void showMessage(const QString& s, int timeout);
       void showHelp(QString);
       void showContextHelp();
       void showHelp(const QUrl&);
 
+      TourHandler* tourHandler()       { return _tourHandler; }
+
       void registerPlugin(PluginDescription*);
       void unregisterPlugin(PluginDescription*);
 
       Q_INVOKABLE void showStartcenter(bool);
+      void reDisplayDockWidget(QDockWidget* widget, bool visible);
       void showPlayPanel(bool);
 
       QFileInfoList recentScores() const;
@@ -692,11 +842,45 @@ class MuseScore : public QMainWindow, public MuseScoreCore {
       LoginManager* loginManager()     { return _loginManager; }
       QHelpEngine*  helpEngine() const { return _helpEngine;   }
 
-      void updateInspector();
+      virtual void updateInspector() override;
+      void updateInstrumentDialog();
+      void reloadInstrumentTemplates();
       void showSynthControl(bool);
       void showMixer(bool);
 
-      qreal physicalDotsPerInch() const { return _physicalDotsPerInch; }
+      qreal physicalDotsPerInch() const                              { return _physicalDotsPerInch; }
+      static const std::list<const char*>& allNoteInputMenuEntries() { return _allNoteInputMenuEntries; }
+      std::list<const char*>* noteInputMenuEntries()                 { return &_noteInputMenuEntries; }
+      void setNoteInputMenuEntries(std::list<const char*> l)         { _noteInputMenuEntries = l; }
+      void populateNoteInputMenu();
+
+      static const std::list<const char*>& allFileOperationEntries() { return _allFileOperationEntries; }
+      std::list<const char*>* fileOperationEntries()                 { return &_fileOperationEntries; }
+      void setFileOperationEntries(std::list<const char*> l)         { _fileOperationEntries = l; }
+      void populateFileOperations();
+
+      static const std::list<const char*>& allPlaybackControlEntries() { return _allPlaybackControlEntries; }
+      std::list<const char*>* playbackControlEntries()               { return &_playbackControlEntries; }
+      void setPlaybackControlEntries(std::list<const char*> l)       { _playbackControlEntries = l; }
+      void populatePlaybackControls();
+
+      bool playPartOnly() const { return _playPartOnly; }
+      void setPlayPartOnly(bool val);
+
+      static void updateUiStyleAndTheme();
+
+      void showError();
+
+      static void saveGeometry(QWidget const*const qw);
+      static void restoreGeometry(QWidget*const qw);
+
+      void updateWindowTitle(Score* score);
+      bool importExtension(QString path);
+      bool uninstallExtension(QString extensionId);
+      Q_INVOKABLE bool isInstalledExtension(QString extensionId);
+
+      ScriptRecorder* getScriptRecorder();
+      bool runTestScripts(const QStringList& scripts);
       };
 
 extern MuseScore* mscore;
@@ -711,11 +895,28 @@ extern Shortcut* midiActionMap[128];
 extern void loadTranslation(QString fileName, QString localeName);
 extern void setMscoreLocale(QString localeName);
 extern bool saveMxl(Score*, const QString& name);
+extern bool saveMxl(Score*, QIODevice*);
+extern bool saveXml(Score*, QIODevice*);
 extern bool saveXml(Score*, const QString& name);
 
 struct PluginDescription;
 extern void collectPluginMetaInformation(PluginDescription*);
 extern QString getSharePath();
 
+extern Score::FileError importMidi(MasterScore*, const QString& name);
+extern Score::FileError importGTP(MasterScore*, const QString& name);
+extern Score::FileError importBww(MasterScore*, const QString& path);
+extern Score::FileError importMusicXml(MasterScore*, const QString&);
+extern Score::FileError importCompressedMusicXml(MasterScore*, const QString&);
+extern Score::FileError importMuseData(MasterScore*, const QString& name);
+extern Score::FileError importLilypond(MasterScore*, const QString& name);
+extern Score::FileError importBB(MasterScore*, const QString& name);
+extern Score::FileError importCapella(MasterScore*, const QString& name);
+extern Score::FileError importCapXml(MasterScore*, const QString& name);
+extern Score::FileError readScore(MasterScore* score, QString name, bool ignoreVersionError);
+
 } // namespace Ms
+
+extern Ms::Score::FileError importOve(Ms::MasterScore*, const QString& name);
+
 #endif
